require 'flashsdk'

module SproutPunk
  module Builds
    def self.config
      _config = SproutPunk.config
      config_path = File.join(_config.root_directory, _config.builds)
      @_config ||= YAML.load_file(config_path)
    end

    class Build
      def initialize(name)
        super
        @name = name
        @config = Builds.config
        Dir.chdir(SproutPunk.config.root_directory)
      end

      def execute
        build_data = @config["packages"][@name]

        pack_exe = package_executable(@name)
        swf_exe  = swf_executable(build_data["swf"])
        cert_exe = cert_executable(build_data["cert"])

        swf_exe.execute
        cert_exe.execute
        pack_exe.execute
      end

      def package_executable(name)
        exe = FlashSDK::ADT.new

        package = @config["packages"][@name].dup
        cert    = @config["certs"][package.delete("cert")]
        swf     = package.delete "swf"

        package.each do |k, v|
          exe.send("#{k}=", v)
        end

        exe.keystore = cert["file"]
        exe.storepass = File.read(cert["password"]).to_s.strip
        exe.included_files << "bin/#{swf}.swf"

        exe
      end

      def swf_executable(name)
        exe = FlashSDK::MXMLC.new

        swf = @config["swfs"][name].dup

        swf.each do |k, v|
          exe.send("#{k}=", v)
        end

        exe.output = "bin/#{name}.swf"

        exe
      end

      def cert_executable(name)
        exe = FlashSDK::ADT.new

        cert = @config["certs"][name].dup

        exe.certificate = true
        exe.cn = 'SelfCertificate'
        exe.key_type = '2048-RSA'
        exe.pfx_file = cert["file"]
        exe.password = File.read(cert["password"]).to_s.strip

        exe
      end
    end
  end
end
