require 'yaml'

module SproutPunk
  class ConfigFile

    def self.default_path
      nil
    end

    def default_path
      self.class.default_path
    end

    def self.load(path = self.default_path)
      new.load(path)
    end

    def initialize(hash = {})
      @table = hash
    end

    def load(path = self.default_path)
      check_path(path)
      @table = YAML.load_file(path) || {}
      self.config_path = path
      self
    end

    def write(path = self.default_path)
      check_path(path)
      File.open(path, 'w') {|f| f << YAML.dump(@table) }
      path
    end

    def to_hash
      @table.dup
    end

    def root_dir
      File.expand_path(@table['root_dir'] || '')
    end

    def inspect
      "#<SproutPunk::ConfigFile #{@table.inspect}>"
    end

    def config_dir
      File.dirname(set?(:config_path) ? config_path : self.class.default_path)
    end

    def set?(key)
      @table.has_key?(key.to_s)
    end

    def method_missing(meth, *args)
      name = meth.to_s
      if name =~ /(.*)\=$/
        @table[$1] = args.first
      elsif @table.has_key?(name)
        val = @table[name]
        val.is_a?(Hash) ? @table[name] = self.class.new(val) : val
      else
        super
      end
    end

    private
    def check_path(path)
      if !File.readable?(path)
        FileUtils.mkdir_p(File.dirname(path))
        FileUtils.touch(path)
      end
    end

  end
end
